Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var BWEFactories = map[string]BWEFactory{ "nada": BWEFactoryFunc(func(config BWEConfig) (mrtp.BWE, error) { nadaConfig := nada.Config{ MinRate: uint64(config.MinTargetRate), MaxRate: uint64(config.MaxTargetRate), StartRate: uint64(config.InitTargetRate), FeedbackDelta: uint64(20 / time.Millisecond), DeactivateQDelayWrapping: true, RefCongLevel: DefaultBweFlags.RefCongLevel, QEPS: DefaultBweFlags.QEPS, MaxRampUpFactor: DefaultBweFlags.MaxRampUpFactor, MaxGradualUpdateFactor: DefaultBweFlags.MaxGradualUpdateFactor, Kappa: DefaultBweFlags.Kappa, Eta: DefaultBweFlags.Eta, DFILT: DefaultBweFlags.DFILT, QBOUND: DefaultBweFlags.QBOUND, } return mrtp.NewNadaWithConfig(nadaConfig), nil }), "gcc": BWEFactoryFunc(func(config BWEConfig) (mrtp.BWE, error) { return mrtp.NewGCC(config.InitTargetRate, config.MinTargetRate, config.MaxTargetRate) }), }
var DefaultBweFlags *bweFlags = &bweFlags{}
var WebRTCExtraCodecs = []WebRTCCodecParameters{}
Functions ¶
This section is empty.
Types ¶
type BWEFactoryFunc ¶
type BitrateAdapter ¶
type BitrateAdapter interface {
// SetBitrate sets the target bitrate for the stream source.
SetBitrate(uint) error
}
BitrateAdapter is the interface implemented by source streams that can adapt their bitrate to a target rate.
type MoQPub ¶
type MoQPub struct {
// contains filtered or unexported fields
}
type MoQSub ¶
type MoQSub struct {
// contains filtered or unexported fields
}
type ReceiveData ¶
type ReceiveData struct {
// contains filtered or unexported fields
}
ReceiveData is a command to run a receiver pipeline for data channels.
func (*ReceiveData) Help ¶
func (r *ReceiveData) Help() string
type SendData ¶
type SendData struct {
// contains filtered or unexported fields
}
SendData is a command to run a receiver pipeline for data channels.
type SendGo ¶
type SendGo struct {
// contains filtered or unexported fields
}
type StreamSinkFactory ¶
type StreamSinkFactory interface {
ConfigureFlags(*flag.FlagSet)
MakeStreamSink(name string, payloadType int) (gstreamer.RTPSinkBin, error)
}
var DefaultStreamSinkFactory StreamSinkFactory = &gstreamerVideoStreamSinkFactory{}
type StreamSourceFactory ¶
type StreamSourceFactory interface {
ConfigureFlags(*flag.FlagSet)
MakeStreamSource(name string) (gstreamer.RTPSourceBin, error)
}
var DefaultStreamSourceFactory StreamSourceFactory = &gstreamerVideoStreamSourceFactory{}
type Version ¶
type Version struct {
// contains filtered or unexported fields
}
type WebRTCCodecParameters ¶
Click to show internal directories.
Click to hide internal directories.